import { Directive, ElementRef, Injectable, InjectionToken, Input, NgModule, Renderer2 } from '@angular/core';
import { Subject } from 'rxjs/Subject';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/observable/from';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/take';
import 'rxjs/add/operator/delay';
import 'rxjs/add/operator/skipWhile';

class HighlightService {
    /**
     * @param {?=} theme
     * @param {?=} path
     */
    constructor(theme = 'github', path = 'assets/lib/hljs') {
        this.theme = theme;
        this.path = path;
    }
}
HighlightService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
HighlightService.ctorParameters = () => [
    null,
    null,
];

class HighlightDirective {
    /**
     * @param {?} el
     * @param {?} renderer
     * @param {?} hl
     */
    constructor(el, renderer, hl) {
        this.renderer = renderer;
        this.hl = hl;
        this.highlighter$ = new Subject();
        this.hlAuto = true;
        this.hlDelay = 200;
        this.el = el.nativeElement;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        let /** @type {?} */ codeElements = [];
        this.highlighter$
            .skipWhile(() => this.notLoaded())
            .delay(this.hlDelay)
            .switchMap(() => {
            switch (this.highlight) {
                case 'all':
                    codeElements = this.el.querySelectorAll('pre code');
                    break;
                case '':
                    codeElements = [this.el];
                    break;
                default:
                    codeElements = this.el.querySelectorAll(this.highlight);
            }
            return Observable.from(codeElements)
                .take(1)
                .map((code) => {
                /** Highlight only If content is plain text */
                if (code.childNodes.length === 1 && code.childNodes[0].nodeName === '#text') {
                    const /** @type {?} */ highlightedCode = hljs.highlightAuto(code.innerText.trim()).value;
                    /** Render the highlighted code */
                    if (highlightedCode !== code.innerHTML) {
                        this.renderer.setProperty(code, 'innerHTML', highlightedCode);
                    }
                }
            });
        }).subscribe();
        /** Load highlight.js script and theme */
        if (this.notLoaded()) {
            this.loadScript();
            this.loadTheme();
        }
        else {
            this.highlighter$.next();
        }
        /** Auto highlight on changes */
        if (this.hlAuto) {
            this.domObs = new MutationObserver(() => this.highlighter$.next());
            this.domObs.observe(this.el, { childList: true, subtree: true });
        }
    }
    /**
     * @return {?}
     */
    loadScript() {
        const /** @type {?} */ script = this.renderer.createElement('script');
        script.async = true;
        script.type = 'text/javascript';
        script.onload = () => {
            this.highlighter$.next();
        };
        script.src = `${this.hl.path}/highlight.pack.js`;
        this.renderer.setAttribute(script, 'data-timestamp', new Date().getTime().toString());
        this.renderer.appendChild(this.el, script);
    }
    /**
     * @return {?}
     */
    loadTheme() {
        const /** @type {?} */ style = this.renderer.createElement('link');
        style.rel = 'stylesheet';
        style.type = 'text/css';
        // style.onload = () => {
        //   console.log('theme loaded');
        // };
        style.href = `${this.hl.path}/styles/${this.hl.theme}.css`;
        this.renderer.appendChild(this.el, style);
    }
    /**
     * @return {?}
     */
    notLoaded() {
        return typeof hljs === 'undefined';
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.hlAuto) {
            this.domObs.disconnect();
        }
        this.highlighter$.unsubscribe();
    }
}
HighlightDirective.decorators = [
    { type: Directive, args: [{
                selector: '[highlight]'
            },] },
];
/**
 * @nocollapse
 */
HighlightDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
    { type: HighlightService, },
];
HighlightDirective.propDecorators = {
    'highlight': [{ type: Input, args: ['highlight',] },],
    'hlAuto': [{ type: Input },],
    'hlDelay': [{ type: Input },],
};

/**
 * Initialize HighlightService with theme and path
 * @param {?} theme
 * @param {?} path
 * @return {?}
 */
function HighlightFactory(theme, path) {
    return new HighlightService(theme, path);
}
const PATH = new InjectionToken('path');
const THEME = new InjectionToken('theme');
class HighlightModule {
    /**
     * @param {?=} theme
     * @param {?=} path
     * @return {?}
     */
    static forRoot(theme, path) {
        return {
            ngModule: HighlightModule,
            providers: [
                { provide: THEME, useValue: theme },
                { provide: PATH, useValue: path },
                {
                    provide: HighlightService,
                    useFactory: HighlightFactory,
                    deps: [THEME, PATH]
                }
            ]
        };
    }
}
HighlightModule.decorators = [
    { type: NgModule, args: [{
                declarations: [HighlightDirective],
                exports: [HighlightDirective]
            },] },
];
/**
 * @nocollapse
 */
HighlightModule.ctorParameters = () => [];

class HighlightUmdDirective {
    /**
     * @param {?} el
     * @param {?} renderer
     */
    constructor(el, renderer) {
        this.renderer = renderer;
        this.highlighter$ = new Subject();
        this.hlAuto = true;
        this.hlDelay = 200;
        this.el = el.nativeElement;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        let /** @type {?} */ codeElements = [];
        this.highlighter$
            .skipWhile(() => this.notLoaded())
            .delay(this.hlDelay)
            .switchMap(() => {
            switch (this.highlight) {
                case 'all':
                    codeElements = this.el.querySelectorAll('pre code');
                    break;
                case '':
                    codeElements = [this.el];
                    break;
                default:
                    codeElements = this.el.querySelectorAll(this.highlight);
            }
            return Observable.from(codeElements)
                .take(1)
                .map((code) => {
                /** Highlight only If content is plain text */
                if (code.childNodes.length === 1 && code.childNodes[0].nodeName === '#text') {
                    const /** @type {?} */ highlightedCode = hljs.highlightAuto(code.innerText.trim()).value;
                    /** Render the highlighted code */
                    if (highlightedCode !== code.innerHTML) {
                        this.renderer.setProperty(code, 'innerHTML', highlightedCode);
                    }
                }
            });
        }).subscribe();
        this.highlighter$.next();
        /** Auto highlight on changes */
        if (this.hlAuto) {
            this.domObs = new MutationObserver(() => this.highlighter$.next());
            this.domObs.observe(this.el, { childList: true, subtree: true });
        }
    }
    /**
     * @return {?}
     */
    notLoaded() {
        return typeof hljs === 'undefined';
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.hlAuto) {
            this.domObs.disconnect();
        }
        this.highlighter$.unsubscribe();
    }
}
HighlightUmdDirective.decorators = [
    { type: Directive, args: [{
                selector: '[highlight]'
            },] },
];
/**
 * @nocollapse
 */
HighlightUmdDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
HighlightUmdDirective.propDecorators = {
    'highlight': [{ type: Input, args: ['highlight',] },],
    'hlAuto': [{ type: Input },],
    'hlDelay': [{ type: Input },],
};

class HighlightUmdModule {
}
HighlightUmdModule.decorators = [
    { type: NgModule, args: [{
                declarations: [HighlightUmdDirective],
                exports: [HighlightUmdDirective]
            },] },
];
/**
 * @nocollapse
 */
HighlightUmdModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { HighlightModule, HighlightUmdModule, HighlightUmdDirective as ɵf, HighlightDirective as ɵd, HighlightFactory as ɵa, PATH as ɵb, THEME as ɵc, HighlightService as ɵe };
//# sourceMappingURL=ngx-highlightjs.js.map
